@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 /*
-  $Id: ruli_res.c,v 1.33 2004/06/17 07:21:01 evertonm Exp $
+  $Id: ruli_res.c,v 1.34 2004/06/21 00:22:38 evertonm Exp $
   */
 
 
@@ -315,6 +315,14 @@ int ruli_res_new(ruli_res_t *res_ctx)
 
 void ruli_res_delete(ruli_res_t *res_ctx)
 {
+#ifdef RULI_RES_DEBUG
+  fprintf(stderr, 
+	  "DEBUG: %s: %s(): %d: "
+	  "query_list_size=%d\n", 
+	  __FILE__, __PRETTY_FUNCTION__, __LINE__,
+	  ruli_list_size(&res_ctx->query_list));
+#endif /* RULI_RES_DEBUG */
+
   /*
    * Clean UDP socket
    */
@@ -484,10 +492,6 @@ int ruli_res_query_submit(ruli_res_t *res_ctx, ruli_res_query_t *res_qry)
 {
   int result;
 
-  /* An option type could avoid this... */
-  assert(res_qry->q_options >= RULI_RES_OPT_VOID);
-  assert(res_qry->q_options <= RULI_RES_OPT_SEARCH);
-
   /* Initially use the domain name without search list */
   assert(res_qry->q_domain_len <= RULI_LIMIT_DNAME_ENCODED);
   memcpy(res_qry->full_dname, res_qry->q_domain, res_qry->q_domain_len);
@@ -520,6 +524,13 @@ int ruli_res_query_submit(ruli_res_t *res_ctx, ruli_res_query_t *res_qry)
   res_qry->remaining_retries = res_ctx->res_retry;
   res_qry->search_index      = -1;
 
+#ifdef RULI_RES_DEBUG
+  fprintf(stderr, 
+	  "DEBUG: %s: %s(): %d: query_id=%d\n", 
+	  __FILE__, __PRETTY_FUNCTION__, __LINE__,
+	  res_qry->query_id);
+#endif /* RULI_RES_DEBUG */
+
   return start_query(res_qry);
 }
 
@@ -529,10 +540,12 @@ void ruli_res_query_delete(ruli_res_query_t *res_qry)
   ruli_list_t *q_list;                      /* resolver query list */
   int         q_ind;                        /* query index in resolver */
 
-  /*
-   * If query is not attached to a resolver, it can't be deleted.
-   */
-  assert(res);
+#ifdef RULI_RES_DEBUG
+  fprintf(stderr, 
+	  "DEBUG: %s: %s(): %d: BEFORE delete: query_id=%d attached_res=%d\n", 
+	  __FILE__, __PRETTY_FUNCTION__, __LINE__,
+	  res_qry->query_id, res != 0);
+#endif /* RULI_RES_DEBUG */
   
   q_list = &res->query_list;
   q_ind  = res_qry->resolver_index;
@@ -553,6 +566,12 @@ void ruli_res_query_delete(ruli_res_query_t *res_qry)
     res_qry->answer_code     = RULI_CODE_VOID;
   }
 
+  /*
+   * If query is not attached to a resolver, it can't be deleted.
+   */
+  if (!res)
+    return;
+
   /*
    * Erase query from the resolver list.
    */
@@ -570,6 +589,13 @@ void ruli_res_query_delete(ruli_res_query_t *res_qry)
    * Mark as detached from resolver.
    */
   res_qry->resolver = 0;
+
+#ifdef RULI_RES_DEBUG
+  fprintf(stderr, 
+	  "DEBUG: %s: %s(): %d: AFTER delete: query_list_size=%d\n", 
+	  __FILE__, __PRETTY_FUNCTION__, __LINE__,
+	  ruli_list_size(q_list));
+#endif /* RULI_RES_DEBUG */
 }
 
 int ruli_res_get_curr_server_index(ruli_res_query_t *qry)
