@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 /*
-  $Id: ruli_res.h,v 1.23 2004/06/17 07:21:01 evertonm Exp $
+  $Id: ruli_res.h,v 1.24 2004/06/21 00:22:39 evertonm Exp $
   */
 
 
@@ -116,11 +116,14 @@ enum {
  * Query options
  */
 enum {
-  RULI_RES_OPT_VOID        = 0,
-  RULI_RES_OPT_SEARCH      = 1 << 0, /* Use search list */
-  RULI_RES_OPT_NEED_RA     = 1 << 1, /* Require RA bit in answer */
-  RULI_RES_OPT_SRV_NOINET  = 1 << 2, /* Don't search IPv4 addresses */
-  RULI_RES_OPT_SRV_NOINET6 = 1 << 3  /* Don't search IPv6 addresses */
+  RULI_RES_OPT_VOID         = 0,
+  RULI_RES_OPT_SEARCH       = 1 << 0, /* Use search list */
+  RULI_RES_OPT_NEED_RA      = 1 << 1, /* Require RA bit in answer */
+  RULI_RES_OPT_SRV_NOINET   = 1 << 2, /* Don't search IPv4 addresses */
+  RULI_RES_OPT_SRV_NOINET6  = 1 << 3, /* Don't search IPv6 addresses */
+  RULI_RES_OPT_SRV_URI_PORT = 1 << 4, /* Assume port was forced in URI */
+  RULI_RES_OPT_SRV_NOWALK   = 1 << 5, /* Don't search missing addresses */
+  RULI_RES_OPT_SRV_NOFALL   = 1 << 6  /* Don't launch fallback queries */
 };
 
 typedef struct {
