@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 /*
-  $Id: ruli_sock.c,v 1.10 2004/05/31 06:14:45 evertonm Exp $
+  $Id: ruli_sock.c,v 1.12 2004/06/16 21:23:11 evertonm Exp $
   */
 
 
@@ -65,7 +65,7 @@ int ruli_sock_create(int family, int type, const char *proto_name)
     struct sockaddr_in inet;
     struct sockaddr_in6 inet6;
   } sa;
-  unsigned int sa_len;
+  unsigned int sa_len = 0; /* picky compilers */
   int proto;
   int sd;
   long flags;
@@ -240,8 +240,8 @@ int ruli_sock_connect(int sd, const ruli_addr_t *remote_addr, int remote_port)
   ruli_sock_set_address((struct sockaddr *) &sa, &sa_len,
 			remote_addr, remote_port);
 
-  assert(sa_len >= sizeof(sa.inet));
-  assert(sa_len <= sizeof(sa.inet6));
+  assert((size_t) sa_len >= sizeof(sa.inet));
+  assert((size_t) sa_len <= sizeof(sa.inet6));
 
   result = connect(sd, (struct sockaddr *) &sa, sa_len);
 
@@ -272,8 +272,8 @@ int ruli_sock_sendto(int sd, const ruli_addr_t *rem_addr, int rem_port,
 
   ruli_sock_set_address((struct sockaddr *) &sa, &sa_len, rem_addr, rem_port);
 
-  assert(sa_len >= sizeof(sa.inet));
-  assert(sa_len <= sizeof(sa.inet6));
+  assert((size_t) sa_len >= sizeof(sa.inet));
+  assert((size_t) sa_len <= sizeof(sa.inet6));
 
 #ifdef RULI_RES_DEBUG
   fprintf(stderr, 
