@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 /*
-  $Id: ruli_srv.c,v 1.68 2004/06/17 14:41:05 evertonm Exp $
+  $Id: ruli_srv.c,v 1.71 2004/06/23 22:10:56 evertonm Exp $
   */
 
 
@@ -36,6 +36,7 @@ Boston, MA 02111-1307, USA.
 #include <ruli_rand.h>
 #include <ruli_txt.h>
 #include <ruli_mem.h>
+#include <ruli_host.h>
 
 
 const char *ruli_srv_errstr(int result)
@@ -81,9 +82,12 @@ static void f_query_done(ruli_srv_t *srv_qry) {
 static void *fall_query_done(ruli_srv_t *srv_qry, int srv_result_code)
 {
 #ifdef RULI_SRV_DEBUG
-      fprintf(stderr, 
-	      "DEBUG: fall_query_done(): query_result=%s [%d]\n", 
-	      srv_result_code ? "FAIL" : "SUCCESS", srv_result_code);
+  fprintf(stderr, 
+	  "DEBUG: %s: %s(): %d: id=%d query_result=%s [%d]\n", 
+	  __FILE__, __PRETTY_FUNCTION__, __LINE__,
+	  srv_qry->fall_query->query_id,
+	  srv_result_code ? "FAIL" : "SUCCESS", 
+	  srv_result_code);
 #endif
 
   f_query_done(srv_qry);
@@ -109,9 +113,12 @@ static void w_query_done(ruli_srv_t *srv_qry)
 static void *walk_query_done(ruli_srv_t *srv_qry, int srv_result_code)
 {
 #ifdef RULI_SRV_DEBUG
-      fprintf(stderr, 
-	      "DEBUG: walk_query_done(): query_result=%s [%d]\n", 
-	      srv_result_code ? "FAIL" : "SUCCESS", srv_result_code);
+  fprintf(stderr, 
+	  "DEBUG: %s: %s(): %d: id=%d query_result=%s [%d]\n", 
+	  __FILE__, __PRETTY_FUNCTION__, __LINE__,
+	  srv_qry->walk_query->query_id,
+	  srv_result_code ? "FAIL" : "SUCCESS", 
+	  srv_result_code);
 #endif
 
   w_query_done(srv_qry);
@@ -119,22 +126,6 @@ static void *walk_query_done(ruli_srv_t *srv_qry, int srv_result_code)
   return query_done(srv_qry, srv_result_code);
 }
 
-static int need_query_in_a(int qtype, long options)
-{
-  return ( (qtype == RULI_RR_TYPE_AAAA) &&
-	   !(options & RULI_RES_OPT_SRV_NOINET) );
-}
-
-int _ruli_reset_qtype(long options)
-{
-  /* can't disable all address families */
-  assert( !((options & RULI_RES_OPT_SRV_NOINET) &&
-	    (options & RULI_RES_OPT_SRV_NOINET6)) );
-
-  return (options & RULI_RES_OPT_SRV_NOINET6) ? 
-    RULI_RR_TYPE_A : RULI_RR_TYPE_AAAA;
-}
-
 static void *on_walk_answer(ruli_res_query_t *qry, void *qry_arg)
 {
   ruli_srv_t       *srv_qry = (ruli_srv_t *) qry_arg;
@@ -196,7 +187,7 @@ static void *on_walk_answer(ruli_res_query_t *qry, void *qry_arg)
 
     if (rcode != RULI_RCODE_NOERROR) {
 
-      if (need_query_in_a(srv_qry->walk_qtype, srv_qry->srv_options)) {
+      if (_ruli_need_query_in_a(srv_qry->walk_qtype, srv_qry->srv_options)) {
 	/* Dispose auxiliary query as it will be re-created
 	   by next walk query */
 	w_query_done(srv_qry);
@@ -318,16 +309,16 @@ static void *on_walk_answer(ruli_res_query_t *qry, void *qry_arg)
 	assert(!result);
 	
 	fprintf(stderr, 
-		"DEBUG: %s: %s(): "
+		"DEBUG: %s: %s(): %d: "
 		"target=%s len=%d w_qtype=%d w_index=%d: "
 		"BUT: no matching address record\n", 
-		__FILE__, __PRETTY_FUNCTION__,
+		__FILE__, __PRETTY_FUNCTION__, __LINE__,
 		target_txt, target_txt_len, 
 		srv_qry->walk_qtype, srv_qry->walk_index);
       }
 #endif
 
-      if (need_query_in_a(srv_qry->walk_qtype, srv_qry->srv_options)) {
+      if (_ruli_need_query_in_a(srv_qry->walk_qtype, srv_qry->srv_options)) {
 	/* Dispose auxiliary query as it will be re-created
 	   by next walk query */
 	w_query_done(srv_qry);
@@ -341,7 +332,7 @@ static void *on_walk_answer(ruli_res_query_t *qry, void *qry_arg)
     }
   }
 
-  if (need_query_in_a(srv_qry->walk_qtype, srv_qry->srv_options)) {
+  if (_ruli_need_query_in_a(srv_qry->walk_qtype, srv_qry->srv_options)) {
     /* Dispose auxiliary query as it will be re-created
        by next walk query */
     w_query_done(srv_qry);
@@ -406,6 +397,10 @@ void *_ruli_srv_answer_walk(ruli_srv_t *srv_qry)
   ruli_list_t *srv_list     = &srv_qry->answer_srv_list;
   int         srv_list_size = ruli_list_size(srv_list);
 
+  /* Have the user disabled walk query? */
+  if (srv_qry->srv_options & RULI_RES_OPT_SRV_NOWALK)
+    return query_done(srv_qry, RULI_SRV_CODE_OK);
+
   /*
    * Scan answer targets, considering address lists
    */
@@ -430,7 +425,8 @@ void *_ruli_srv_answer_walk(ruli_srv_t *srv_qry)
       assert(!result);
       
       fprintf(stderr, 
-	      "DEBUG: _ruli_srv_answer_walk(): missing target=%s walk_index=%d walk_qtype=%d\n", 
+	      "DEBUG: _ruli_srv_answer_walk(): "
+              "missing target=%s walk_index=%d walk_qtype=%d\n", 
 	      target_txt, srv_qry->walk_index, srv_qry->walk_qtype);
     }
 #endif
@@ -481,13 +477,20 @@ void *_ruli_srv_answer_walk(ruli_srv_t *srv_qry)
 
 static void *fallback_resubmit(ruli_srv_t *srv_qry)
 {
+#ifdef RULI_SRV_DEBUG
+  fprintf(stderr, 
+	  "DEBUG: %s: %s(): %d: OLD query_id=%d\n", 
+	  __FILE__, __PRETTY_FUNCTION__, __LINE__,
+	  srv_qry->fall_query->query_id);
+#endif /* RULI_SRV_DEBUG */
+
   /* reset fallback query, but not dispose it */
   assert(srv_qry->fall_query);
   ruli_res_query_delete(srv_qry->fall_query);
 
   srv_qry->fall_query->q_type = srv_qry->fall_qtype;
 
-  assert(!need_query_in_a(srv_qry->fall_qtype, srv_qry->srv_options));
+  assert(!_ruli_need_query_in_a(srv_qry->fall_qtype, srv_qry->srv_options));
 
   if (ruli_res_query_submit(srv_qry->srv_resolver, srv_qry->fall_query)) {
     ruli_free(srv_qry->fall_query);
@@ -495,6 +498,13 @@ static void *fallback_resubmit(ruli_srv_t *srv_qry)
     return query_done(srv_qry, RULI_SRV_CODE_FALL_OTHER);
   }
 
+#ifdef RULI_SRV_DEBUG
+  fprintf(stderr, 
+	  "DEBUG: %s: %s(): %d: NEW query_id=%d\n", 
+	  __FILE__, __PRETTY_FUNCTION__, __LINE__,
+	  srv_qry->fall_query->query_id);
+#endif /* RULI_SRV_DEBUG */
+
   return OOP_CONTINUE;
 }
 
@@ -520,11 +530,13 @@ static void *on_fallback_answer(ruli_res_query_t *qry, void *qry_arg)
     assert(!result);
 
     fprintf(stderr, 
-	    "DEBUG: on_fallback_answer(): domain=%s len=%d f_qtype=%d\n",
+	    "DEBUG: %s: %s(): %d: domain=%s len=%d f_qtype=%d\n",
+	    __FILE__, __PRETTY_FUNCTION__, __LINE__,
 	    txt_dname_buf, txt_dname_len, srv_qry->fall_qtype);
 
     fprintf(stderr, 
-	    "DEBUG: on_fallback_answer(): id=%d answer_code=%d rcode=%d\n",
+	    "DEBUG: %s: %s(): %d: id=%d answer_code=%d rcode=%d\n",
+	    __FILE__, __PRETTY_FUNCTION__, __LINE__,
 	    qry->query_id, qry->answer_code, qry->answer_header.rcode);
   }
 #endif
@@ -555,7 +567,7 @@ static void *on_fallback_answer(ruli_res_query_t *qry, void *qry_arg)
 
     if (rcode != RULI_RCODE_NOERROR) {
 
-      if (need_query_in_a(srv_qry->fall_qtype, srv_qry->srv_options)) {
+      if (_ruli_need_query_in_a(srv_qry->fall_qtype, srv_qry->srv_options)) {
 	srv_qry->fall_qtype = RULI_RR_TYPE_A;
 	return fallback_resubmit(srv_qry);
       }
@@ -633,8 +645,9 @@ static void *on_fallback_answer(ruli_res_query_t *qry, void *qry_arg)
 	assert(!result);
 	
 	fprintf(stderr, 
-		"DEBUG: on_fallback_answer(): "
+		"DEBUG: %s: %s(): %d: "
 		"wanted owner=(%d)%s answer_list_len=%d\n", 
+		__FILE__, __PRETTY_FUNCTION__, __LINE__,
 		wanted_txt_len, wanted_txt, an_list_size);
       }
 #endif
@@ -718,7 +731,7 @@ static void *on_fallback_answer(ruli_res_query_t *qry, void *qry_arg)
 	if (curr_srv_list_size > 0) {
 	  ruli_parse_delete(&parse);
 
-	  if (need_query_in_a(srv_qry->fall_qtype, srv_qry->srv_options)) {
+	  if (_ruli_need_query_in_a(srv_qry->fall_qtype, srv_qry->srv_options)) {
 	    srv_qry->fall_qtype = RULI_RR_TYPE_A;
 	    return fallback_resubmit(srv_qry);
 	  }
@@ -729,8 +742,9 @@ static void *on_fallback_answer(ruli_res_query_t *qry, void *qry_arg)
 
 #ifdef RULI_SRV_DEBUG
       fprintf(stderr, 
-	      "DEBUG: on_fallback_answer(): id=%d answer_code=%d, rcode=%d: "
+	      "DEBUG: %s: %s(): %d: id=%d answer_code=%d, rcode=%d: "
 	      "BUT: no matching address record\n",
+	      __FILE__, __PRETTY_FUNCTION__, __LINE__,
 	      qry->query_id, qry->answer_code, qry->answer_header.rcode);
 #endif
 
@@ -794,8 +808,9 @@ static void *on_fallback_answer(ruli_res_query_t *qry, void *qry_arg)
 	  assert(!result);
 	  
 	  fprintf(stderr, 
-		  "DEBUG: on_fallback_answer(): "
+		  "DEBUG: %s: %s(): %d: "
 		  "CNAME owner=(%d)%s target=(%d)%s remaining=%d\n", 
+		  __FILE__, __PRETTY_FUNCTION__, __LINE__,
 		  cname_owner_txt_len, cname_owner_txt,
 		  cname_trg_txt_len, cname_trg_txt,
 		  remaining_cname_depth);
@@ -829,7 +844,7 @@ static void *on_fallback_answer(ruli_res_query_t *qry, void *qry_arg)
     
   } /* parsed answer context */
   
-  if (need_query_in_a(srv_qry->fall_qtype, srv_qry->srv_options)) {
+  if (_ruli_need_query_in_a(srv_qry->fall_qtype, srv_qry->srv_options)) {
     srv_qry->fall_qtype = RULI_RR_TYPE_A;
     return fallback_resubmit(srv_qry);
   }
@@ -854,18 +869,26 @@ void *_ruli_srv_answer_fallback_addr(ruli_srv_t *srv_qry)
     assert(!result);
 
     fprintf(stderr, 
-	    "DEBUG: srv_answer_fallback(): address query: domain=%s domain_len=%d\n",
-	    txt_dname_buf, txt_dname_len);
+	    "DEBUG: %s: %s(): %d: address query: "
+	    "domain=%s domain_len=%d fallback=%d\n",
+	    __FILE__, __PRETTY_FUNCTION__, __LINE__,
+	    txt_dname_buf, txt_dname_len, 
+            !!(srv_qry->srv_options & RULI_RES_OPT_SRV_NOFALL));
   }
 #endif
 
   assert(!srv_qry->fall_query);
   assert(srv_qry->fall_qtype == -1);
 
+  /* Has the user disabled fallback query? */
+  if (srv_qry->srv_options & RULI_RES_OPT_SRV_NOFALL)
+    return query_done(srv_qry, RULI_SRV_CODE_OK);
+
   /*
    * Allocate space for fallback query
    */
-  srv_qry->fall_query = (ruli_res_query_t *) ruli_malloc(sizeof(ruli_res_query_t));
+  srv_qry->fall_query = (ruli_res_query_t *) \
+    ruli_malloc(sizeof(ruli_res_query_t));
   if (!srv_qry->fall_query)
     return query_done(srv_qry, RULI_SRV_CODE_FALL_OTHER);
 
@@ -1318,9 +1341,10 @@ static void *on_srv_answer(ruli_res_query_t *qry, void *arg)
     {
       fflush(stdout);
       fprintf(stderr,
-	      "DEBUG: %s: %s(): "
-	      "BEFORE addit inspection: srv_rr_list_size=%d srv_answer_list_size=%d\n",
-	      __FILE__, __PRETTY_FUNCTION__,
+	      "DEBUG: %s: %s(): %d: "
+	      "BEFORE addit inspection: "
+	      "srv_rr_list_size=%d srv_answer_list_size=%d\n",
+	      __FILE__, __PRETTY_FUNCTION__, __LINE__,
 	      src_list_size, ruli_list_size(&srv_qry->answer_srv_list));
       fflush(stderr);
     }
@@ -1395,9 +1419,10 @@ static void *on_srv_answer(ruli_res_query_t *qry, void *arg)
     {
       fflush(stdout);
       fprintf(stderr,
-	      "DEBUG: %s: %s(): "
-	      "AFTER addit inspection: srv_rr_list_size=%d srv_answer_list_size=%d\n",
-	      __FILE__, __PRETTY_FUNCTION__,
+	      "DEBUG: %s: %s(): %d: "
+	      "AFTER addit inspection: "
+	      "srv_rr_list_size=%d srv_answer_list_size=%d\n",
+	      __FILE__, __PRETTY_FUNCTION__, __LINE__,
 	      src_list_size, ruli_list_size(&srv_qry->answer_srv_list));
       fflush(stderr);
     }
@@ -1568,6 +1593,22 @@ int _ruli_srv_query_submit(ruli_srv_t *srv_qry,
   qry->q_type       = RULI_RR_TYPE_SRV;
   qry->q_options    = srv_qry->srv_options;
 
+  /*
+   * If the RULI port is explicitely defined in the URI,
+   * the user wants only address records.
+   *
+   * Skip SRV query and fetch addresses instead.
+   */
+  if (srv_qry->srv_options & RULI_RES_OPT_SRV_URI_PORT) {
+    srv_qry->query.resolver = 0;
+    {
+      void *oop_result = _ruli_srv_answer_fallback_addr(srv_qry);
+      if (oop_result != OOP_CONTINUE)
+	return RULI_SRV_CODE_QUERY_FAILED;
+    }
+    return RULI_SRV_OK;
+  }
+
   /*
    * Submit plain query to underlying resolver
    */
@@ -1589,6 +1630,16 @@ int ruli_srv_query_submit(ruli_srv_t *srv_qry)
 
 void ruli_srv_query_delete(ruli_srv_t *srv_qry)
 {
+#ifdef RULI_SRV_DEBUG
+  fprintf(stderr, 
+	  "DEBUG: %s: %s(): %d: "
+	  "query_id=%d fall_query_id=%d walk_query_id=%d\n", 
+	  __FILE__, __PRETTY_FUNCTION__, __LINE__,
+	  srv_qry->query.query_id,
+	  srv_qry->fall_query ? srv_qry->fall_query->query_id : -1,
+	  srv_qry->walk_query ? srv_qry->walk_query->query_id : -1);
+#endif /* RULI_SRV_DEBUG */
+
   /*
    * Dispose list of srv answers
    */
@@ -1601,7 +1652,8 @@ void ruli_srv_query_delete(ruli_srv_t *srv_qry)
      * For each entry, release list of *ruli_addr_t
      */
     for (i = 0; i < list_size; ++i) {
-      ruli_srv_entry_t *srv_entry = (ruli_srv_entry_t *) ruli_list_get(list, i);
+      ruli_srv_entry_t *srv_entry = (ruli_srv_entry_t *) \
+	ruli_list_get(list, i);
       ruli_list_dispose_trivial(&srv_entry->addr_list);
     }
   }
