@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 /*
-  $Id: ruli_txt.c,v 1.15 2004/05/25 22:52:56 evertonm Exp $
+  $Id: ruli_txt.c,v 1.16 2004/06/16 21:23:11 evertonm Exp $
   */
 
 
@@ -28,14 +28,21 @@ Boston, MA 02111-1307, USA.
 #endif
 
 
-#include <stdio.h>        /* FIXME: remove me [used for fprintf() debug] */
-#include <assert.h>
+#include <strings.h>
 #include <string.h>
+#include <stdio.h>
+#include <assert.h>
 
 #include <ruli_txt.h>
 #include <ruli_limits.h>
 
 
+#ifdef _RULI_SOLARIS_
+/* should be in <strings.h> */
+extern int strncasecmp(const char *s1, const char *s2, size_t n);
+#endif
+
+
 static int build_label(ruli_uint8_t *buf, int buf_len, 
 		       const ruli_uint8_t *label, int label_len)
 {
